from os.path import join
from BeautifulSoup import BeautifulSoup

### Parsing deployment reports

class DeploymentReport:

    def __init__(self, reports_folder, report_name):
        self.report_name = report_name
        self.report_path = join(reports_folder, report_name)
        self.fixed_bugs = []
        self.contents = None

    def fetch_contents(self):
        self.contents = "".join(open(self.report_path).readlines())
        self.bs = BeautifulSoup(self.contents)

    def get_deployment_qa_data(self):
        if not self.contents:
            self.fetch_contents()
        # Get current deployed to qa revno and current number of revnos
        # blocked in QA
        revnos = self.bs.find(name="section",
            attrs={"id":"pending_revisions"}).findAll(name="strong")
        if len(revnos) > 1:
            self.current_deployed_to_qa_revno, self.blocked_revnos_count = revnos
            self.blocked_revnos_count = int(self.blocked_revnos_count.string)
        else:
            self.current_deployed_to_qa_revno = revnos[0]
            self.blocked_revnos_count = 0
        self.current_deployed_to_qa_revno = int(
            self.current_deployed_to_qa_revno.string)

        # Get current blocking revno
        try:
            self.current_blocker_details = {}
            self.current_blocking_revno = int(self.bs.find(name="section",
                attrs={"id":"blocker"}).find(name="strong").string.replace(
                    "Revision ", ""))
            current_blocker_details = self.bs.find(name="table",
                attrs={"id":"blocker_detail"}).find(name="td")
            self.current_blocker_details["bug"] = current_blocker_details.find(
                name="a").contents[0].strip().split()[1]
            self.current_blocker_details["reason"] = current_blocker_details.find(
                name="strong").string
        except:
            self.current_blocking_revno = None
            self.current_blocker_details = None

        # Get current number of revnos ready to deploy (current deployed +
        #   current ready to deploy)
        try:
            self.current_deployable_revno = int(self.bs.find(name="section",
                attrs={"id":"deployable_revision"}).find(
                name="strong").string.replace("Revision ", ""))
        except:
            self.current_deployable_revno = None
            self.current_ready_to_deploy = 0

        all_deployable_revnos = self.bs.find(name="section",
            attrs={"id":"deployments"}).findAll(name="li",
            attrs={"class":"deployable"})

        fixed_bugs = []
        first_deployable_revno = None
        for i in all_deployable_revnos:
            try:
                if not first_deployable_revno:
                    first_deployable_revno = int(
                        i.find(name="a").contents[0].split()[1])
                fixed_bugs.append(i.find(name="div",
                    attrs={"class":"buglinks"}).find(name="a").contents[0])
            except:
                continue

        self.fixed_bugs = fixed_bugs
        if all_deployable_revnos:
            self.current_deployed_revno = first_deployable_revno - 1
        else:
            # XXX: if first deployable revno is none, so the first blocked is
            # the one.
            if self.current_blocking_revno:
                self.current_deployed_revno = self.current_blocking_revno - 1
            else:
                self.current_deployed_revno = self.current_deployed_to_qa_revno
        if self.current_deployable_revno:
            self.current_ready_to_deploy = (
                self.current_deployable_revno - self.current_deployed_revno)
        else:
            self.current_ready_to_deploy = 0

    def list_deployable_bugs(self):
        return " ".join(self.fixed_bugs)


#dr_stable = DeploymentReport("deployment-stable.html")
#dr_stable.get_deployment_qa_data()
#
#dr_dbstable = DeploymentReport("deployment-db-stable.html")
#dr_dbstable.get_deployment_qa_data()
