from editmoin import editshortcut
from datetime import datetime, timedelta
from BeautifulSoup import BeautifulSoup
from devops.utils import days_ago, last_date_delta

### Find last LP incident report


class Wiki:
    def __init__(self, url):
        self.url = url
        self.contents = []
        self.data = []

    def read(self):
        def editfunc(moinfile):
            if "This page was opened for editing" in moinfile.data:
                # Cancel the editing.
                return 0
            self.read_contents(moinfile)
            return 1

        if not editshortcut(self.url, action="print", editfile_func=editfunc,
            change_page=False):
            # Someone else was editing the page. Raise an exception
            raise Exception("Couldn't get write lock for the wiki page. "
                "Aborting.")

    def read_contents(self, moinfile):
        self.data = moinfile._unescape(moinfile.data)


class Reports:

    def __init__(sel, source_url):
        self.source_url = source_url

    def fetch_data(self):
        raise NotImplementedError


class IncidentReports:

    def __init__(self, since=datetime(2011, 01, 01)):
        self.source_url = "https://wiki.canonical.com/IncidentReports"
        self.all_reports = []
        # { Date : report}
        self.lp_reports = {}
        self.most_days_with_no_incidents = 0
        self._since = since
        self.last_report = None
        self._last_date = None

    def format_date(self, date_):
        return datetime.strftime(date_, "%Y-%m-%d")

    @property
    def since(self):
        return self.format_date(self._since)

    @property
    def last_date(self):
        if self._last_date:
            return self.format_date(self._last_date)
        return None

    @property
    def _days_ago(self):
        if self.last_date:
            return last_date_delta(self.last_date)
        return None

    @property
    def days_without_incidents(self):
        if self._last_date:
            return days_ago(self._last_date)

    def fetch_reports(self):
        ir_wiki = Wiki(self.source_url)
        ir_wiki.read()
        self.bs = BeautifulSoup(ir_wiki.data)
        self.all_reports = self.bs.findChildren(name="li")

    def get_launchpad_reports(self):
        if self.all_reports == []:
            self.fetch_reports()
        for report in self.all_reports:
            try:
                incident_report_name = report.find(name="a").contents[1]
            except IndexError:
                continue
            if ("LP-" in incident_report_name or
                "GR-" in incident_report_name):
                incident_date_ = self.incident_date(incident_report_name)
                self.lp_reports.setdefault(incident_date_, [])
                self.lp_reports[incident_date_].append(incident_report_name)

    def incident_date(self, incident_report_name):
        report_name = incident_report_name.split("-")
        date_ = "%s-%s-%s" % (report_name[0], report_name[1], report_name[2])
        return datetime.strptime(date_, "%Y-%m-%d")

    def get_last_reports_since(self):
        """Get reports since a given date, default 2011-01-01."""
        if self.lp_reports == {}:
            self.get_launchpad_reports()
        last_reports = []
        for report in sorted(self.lp_reports.iterkeys(), reverse=True):
            if report > self._since:
                last_reports.append(report)

        overall_delta = timedelta(0)

        for i, report in enumerate(sorted(last_reports)):
            current = report
            if i == 0:
                previous = report
                continue
            current_delta = current - previous
            if current_delta > overall_delta:
                overall_delta = current_delta
            previous = current

        self.most_days_with_no_incidents = overall_delta.days
        self.total_incidents = len(last_reports)
        self.last_report = self.lp_reports[current][0]
        self._last_date = current

